//+------------------------------------------------------------------+
//|                                                                  |
//| idea for zero lag hull moving average                            |
//| originaly by spikoloco                                           }
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  DeepSkyBlue
#property indicator_color2  Red
#property indicator_color3  Red
#property indicator_color4  Gold
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_level1 80
#property indicator_level2 20
#property indicator_level3 0

//
//
//
//
//

extern string TimeFrame = "current time frame";
extern int    HMALength = 10;
extern int    KPeriod   = 14;
extern int    DPeriod   = 5;
extern int    Slowing   = 5;
extern int    Method    = 0;
extern int    Price     = PRICE_CLOSE;

//
//
//
//
//

double hma[];
double hmada[];
double hmadb[];
double hma1[];
double hma2[];
double trend[];
double SignalBuffer[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(7);
      SetIndexBuffer(0,hma);
      SetIndexLabel(0,"Stoch Main");
      SetIndexBuffer(1,hmada);
      SetIndexBuffer(2,hmadb);
      SetIndexBuffer(3,SignalBuffer);
      SetIndexLabel(3,"Stoch Signal");
      SetIndexBuffer(4,hma1); 
      SetIndexBuffer(5,hma2); 
      SetIndexBuffer(6,trend); 

      //
      //
      //
      //
      //
   
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame=="returnBars";     if (returnBars)     return(0);
      calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
      
   //
   //
   //
   //
   //
         
   IndicatorShortName(timeFrameToString(timeFrame)+" zero lag HMA Stochastic("+HMALength+","+KPeriod+","+DPeriod+","+Slowing+")");         
   return(0);
}
int deinit() { return(0); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { hma[0] = limit+1; return(0); }
         
   //
   //
   //
   //
   //

   if (calculateValue || timeFrame==Period())
   {
      if (trend[limit] == -1) CleanPoint(limit,hmada,hmadb);
      for(i=limit; i>=0; i--) {
         hma1[i] = iHma(iStochastic(NULL,0,KPeriod,DPeriod,Slowing,Method,Price,MODE_MAIN,i),HMALength,i,0);
         hma2[i] = iHma(hma1[i]                                                             ,HMALength,i,2);
         hma[i]  = 2.0*hma1[i]-hma2[i];
        
         hmada[i] = EMPTY_VALUE;
         hmadb[i] = EMPTY_VALUE;
         trend[i] = trend[i+1];
            if (hma[i] > hma[i+1]) trend[i] =   1;
            if (hma[i] < hma[i+1]) trend[i] =  -1;
            if (trend[i] == -1) PlotPoint(i,hmada,hmadb,hma);
         }
       
         for(i=limit; i>=0; i--) SignalBuffer[i] = iMAOnArray(hma,0,DPeriod,0,Method,i);
         
      return(0);
   }      

   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (trend[limit] == -1) CleanPoint(limit,hmada,hmadb);
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         trend[i]        = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",HMALength,KPeriod,DPeriod,Slowing,Method,Price,6,y);
         hma[i]          = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",HMALength,KPeriod,DPeriod,Slowing,Method,Price,0,y);
         SignalBuffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",HMALength,KPeriod,DPeriod,Slowing,Method,Price,3,y);
         hmada[i] = EMPTY_VALUE;
         hmadb[i] = EMPTY_VALUE;
         

   }
   for(i=limit; i>=0; i--) if (trend[i]==-1) PlotPoint(i,hmada,hmadb,hma);
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double wrk[][4];

//
//
//
//
//

double iHma(double price, int period, int i, int s=0)
{
   if (ArrayRange(wrk,0)!=Bars) ArrayResize(wrk,Bars);
   
   //
   //
   //
   //
   //
   
      int HMAPeriod  = MathMax(2,period);
      int HalfPeriod = MathFloor(HMAPeriod/2);
      int HullPeriod = MathFloor(MathSqrt(HMAPeriod));

      //
      //
      //
      //
      //
         
      int r = Bars-i-1;  
         wrk[r][s  ] = price;
         wrk[r][s+1] = 2.0*iLwma(s,HalfPeriod,r)-iLwma(s,HMAPeriod,r);
      return (iLwma(s+1,HullPeriod,r));
}

//+----------------------------------------------------------------------------------+
//|                                                                                  |
//+----------------------------------------------------------------------------------+
//
//
//
//
//

double iLwma(int forBuffer, int period, int r)
{
   double weight = 0;
   double sum    = 0;
   int    i,k;
   
   if (r>=period)
   {
      for (i=0, k=period; i<period; i++,k--)
      {
            weight += k; sum += wrk[r-i][forBuffer]*k;
      }
      if (weight !=0)
            return(sum/weight);
      else  return(0.0);
    }
    else return(wrk[r][forBuffer]);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}


//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}